/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.result;

import org.springframework.stereotype.Component;

import de.ozgcloud.processor.ProcessorManagerConfiguration;
import de.ozgcloud.processor.processor.ProcessorResult;

@Component(ProcessorManagerConfiguration.AKTENNOTIZ_FACTORY_NAME) // NOSONAR
public class AktennotizFactory {

	static final String AKTENNOTIZ_ERROR_TEXT = """
			Bei der Verarbeitung im Processor '%s' ist ein Fehler aufgetreten.
			ExceptionId: %s
			""";

	public Aktennotiz createFrom(ProcessorResult processorResult) {
		var resultBody = processorResult.getBody();
		return Aktennotiz.builder()
				.vorgangId(processorResult.getVorgangId().toString())
				.severity(resultBody.getSeverity())
				.headline(resultBody.getHeadline())
				.text(resultBody.getText())
				.build();
	}

	public Aktennotiz createErrorAktennotiz(String vorgangId, String processorName, String exceptionId) {
		return createErrorAktennotizBuilder(vorgangId)
				.text(buildErrorText(processorName, exceptionId))
				.build();
	}

	public Aktennotiz.AktennotizBuilder createErrorAktennotizBuilder(String vorgangId) {
		return Aktennotiz.builder()
				.vorgangId(vorgangId)
				.severity(Severity.ERROR)
				.headline("Fehler");
	}

	String buildErrorText(String processorName, String exceptionId) {
		return AKTENNOTIZ_ERROR_TEXT.formatted(processorName, exceptionId);
	}

}
