/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.processor;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.test.ITCase;
import lombok.SneakyThrows;

@ITCase
class ProcessorPropertiesITCase {

	@Autowired
	private ProcessorService service;

	@Test
	void shouldHaveTwoProperties() {
		var props = getProperties();

		assertThat(props.getProcessors()).hasSize(2);
	}

	@Test
	@SneakyThrows
	void shouldHaveFirstAddress() {
		var processors = getProperties().getProcessors();

		assertThat(processors.get(0).getAddress()).isEqualTo("http://test1");
	}

	@Test
	void shouldHaveFormIds() {
		var forms = getProperties().getProcessors().get(0).getForms();

		assertThat(forms.stream().map(ProcessorProperties.Form::getFormId)).contains("ID1", "ID2");
	}

	@Test
	void shouldHaveFormEngineName() {
		var forms = getProperties().getProcessors().get(0).getForms();

		assertThat(forms.stream().map(ProcessorProperties.Form::getFormEngineName)).containsOnly("formEngineTest1");
	}

	private ProcessorProperties getProperties() {
		return (ProcessorProperties) ReflectionTestUtils.getField(service, "properties");
	}

}
